/////////////////////////////////////
//	Fur fragment shader
//	with antialiasing
//
//           by Jonas Nilsson
//       jonni957@student.liu.se
/////////////////////////////////////


uniform int layers; //Total number of layers
uniform int level; //Which layer is the fragment in
uniform float freq; //The frequency of the fur

varying vec3 lightDir,normal,tangent,binorm,halfv; //Used in lighting calculations
varying vec2 v_texCoord2D; //Texture coordinat

void main (void)
{
	//Ambient and diffuse color is calcuated from the material
	vec4 amb = gl_FrontMaterial.ambient*gl_LightSource[0].ambient;
	vec4 dif = gl_FrontMaterial.diffuse*gl_LightSource[0].diffuse;
    vec4 globalAmbient = gl_LightModel.ambient * gl_FrontMaterial.ambient;
	
	//Ambient color is decreased for the lowest layer
	vec4 color = (amb + globalAmbient)*0.5;
	
	//Alpha is set to 1 and the new normal to use is the same as the old
	float alpha = 1.0;		
	vec3 nynorm = normal;
 	
	//The fur calcualtions is not used for the lowest layer
	if( level!=0)
	{
		//The value to be used in the smoothstep is calculated
		//the frequency in s is twice that in t direction for the sphere model used. A frequency calcualtion based
		//on the real postition of the fragment instead of the texture coordinate should be better. 
		float v = abs((sin(freq*2*v_texCoord2D.s)+sin(freq*v_texCoord2D.t)));
	
		//The clipvalue to use in the smoothstep function is calculated
		float clipvalue = 2.0/sqrt(freq);	
		clipvalue = clipvalue*((((float)level-1)/(float)layers));
	
		float pixelbredd  = length(clipvalue*vec4(dFdx(v_texCoord2D), dFdy(v_texCoord2D)));
		
		//Calculaing gradient
		float grads = (sin(freq*2*v_texCoord2D.s)+sin(freq*v_texCoord2D.t)) 
						* 2*freq*cos(freq*2*v_texCoord2D.s) 
						/ abs(sin(freq*v_texCoord2D.t)+sin(freq*2*v_texCoord2D.s));
		
		float gradt = (sin(freq*v_texCoord2D.t)+sin(freq*2*v_texCoord2D.s)) 
						* freq*cos(freq*v_texCoord2D.t) 
						/ abs(sin(freq*v_texCoord2D.t)+sin(freq*2*v_texCoord2D.s));
		//Lenght of gradient
		float gradientL = length(vec2(grads,gradt));
		
		//clipvalue = pixelbredd*((float)level-1)/(float)layers;
		
		//v *= length(vec2(dFdx(v),dFdy(v)));
		
		
		v*=gradientL;
		//The new alpha value is calculated with a smoothstep function
		alpha = smoothstep(clipvalue,1.5*clipvalue,v); 
		
		//The U and V is used to rotate the normal in the direction of the binormal and the tangent.
		//The amount to move the normal is the rate of change of the v function in the x or y direction
		vec3 U = -clamp(dFdx(v),-0.1,0.1) * cross(binorm,nynorm); 
		vec3 V = -clamp(dFdy(v),-0.1,0.1) * cross(tangent,nynorm);
		
		//The new normal is calculated by just adding the U and V vektor to the normal. And normalize the result
		nynorm = normalize(nynorm+U+V);
		
		//A new ambient color is calculated depending on the layer. 
		//This is a simple little effect to amplify the feeling of depth in the fur.
		color = (amb + globalAmbient)*(((float)level/(float)layers));
	}
	
	//Calculating dot product with the new normal. The angle of incomming light
	float NdotL = dot(lightDir,nynorm);
	//If NdotL > 0.0 will incomming light be reflected, 
    //resulting in specular and diffuse
	if (NdotL > 0.0) 
    {
        color+= dif*NdotL;
      	vec3 halfvec = normalize(halfv);
		// normalize halfv and then compute the 
		// cosine (dot product) with the newv normal 
		//Halfvec = vector between lightdirection and camera
		//NdotV angle between halfvector and normal, resulting in specular highlight
		float NdotHV = max(dot(nynorm, halfvec),0.0);
		color += gl_FrontMaterial.specular * 
					gl_LightSource[0].specular * 
					pow(NdotHV, gl_FrontMaterial.shininess)*(((float)level/(float)layers));
	}
	
	//Set the w component of the color to the alpha value
    color.w = alpha;
	
	//And were'r done
	gl_FragColor = color;   	
}	
