/////////////////////////////////////
//            Fur vertex shader
//	with movement of the vertex 
//       according to a windvector
//
//           by Jonas Nilsson
//      jonni957@student.liu.se
/////////////////////////////////////

uniform int layers; //Total number of layers
uniform int level; //Which layer is the fragment in
uniform float time; //The time since the program started
uniform float shelldist; //Distance between shells

//vectors used for light calculation in the fragment shader
varying vec3 lightDir,normal,tangent,binorm,halfv; 

varying vec2 v_texCoord2D; //texture coordinate
attribute vec3 vTangent; //vTangent attribute is not avaliable on all graphic cards

#define PI 3.14159265 //A value for PI

void main()
{	
	//Direction of light 0 and the halfvector for that light is stored in varying variables
    lightDir = normalize(vec3(gl_LightSource[0].position));	
    halfv=gl_LightSource[0].halfVector.xyz;
    
	//Normal, tangent and binormal is stored in varying variables 
	normal = normalize(gl_NormalMatrix * gl_Normal);
    tangent = vTangent;
	binorm = normalize(cross(normal,tangent));
	
	//Texture coordinat is stored in a varying varible
	v_texCoord2D=gl_MultiTexCoord0.st;
	
	//Store vertex coordinate in new vector
	vec4 pos = gl_Vertex;
	
	//Calculate gl_position to get a world koordinat for the vertex 
	gl_Position = gl_ModelViewProjectionMatrix * pos;
	
	//Calculation of wind	
	if(level!=0)
	{
	
	//Intesity of the wind
	 float windint = 30.0;
	 
	//Atempt to create windsource from the lightsource,
	 vec3 windsource = vec3(gl_LightSource[0].position);
	 vec3 w = normalize(gl_Position-windsource);
	 vec3 winddir = normalize(w-dot((dot(w,gl_Normal)),gl_Normal));
	 
	//Create a timevarying winddirection
	//vec3 winddir = normalize(vec3(0,10.0*cos(time*0.1),10.0*sin(time*0.1)));
	
	//Calculate starting position of the vertex
	pos.xyz -= (gl_Normal*level*shelldist);
	
	//Loop for each level up to current level
	float i;
		for(i=0;i<=level;i++)
		{
		//Calculate the distortion in each layer. multiply by the distance between each layer
		pos.xyz  += ((gl_Normal * cos (PI* windint*i*shelldist/ (2.0*layers)))
					+(winddir* sin (PI* windint*shelldist*i/ (2.0*layers))))*shelldist;
		}

	}
	
	//Done
	gl_Position = gl_ModelViewProjectionMatrix * pos;
}